/*
 This file is part of the digitalSTROM ds485-core library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM ds485-core library is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation version 2
 of the License.

 digitalSTROM ds485-core library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM ds485-core library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <signal.h>
#include <setjmp.h>
#include <syslog.h>
#include <time.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <fcntl.h>
#include <unistd.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <semaphore.h>

#include <ds485-interface.h>

#define        LOG_DEV        "/tmp/ds485log"
int        fdLog    = -1;
sem_t    sem;

void sprintDsid( dsid_t *dsid, char* chr )
{
    int    i;
    for( i = 0; i < 12; i++ ){
        sprintf( chr, "%02x ", dsid->value[i] );
        chr += 3;
    }
    chr    = '\0';
}

void callback_bus_change(void* dev, dsid_t *dsid, int flags )
{
    char    buf[40];

    sprintDsid( dsid, buf );

    if( flags == 1 )    printf( "lost     " );
        else            printf( "register " );
    printf( "dsid: %s\n", buf );

    sem_post( &sem );
}

void callback_received(ds485Handle_t dev, Ds485Container_t const * packet)
{
    int idx;
    for( idx = 0; idx < packet->length; idx++ ){
        printf( "%2x ", packet->data[idx] );
    }
    char    buf[40];
    sprintDsid( &packet->address, buf );
    printf( "\nfrom %s\n", buf );
}

void callback_sender(ds485Handle_t dev, int error)
{
    printf( "message sent: %i\n", error );
}

void callback_state( ds485Handle_t dev, BusState state )
{
    printf( "state: %i\n", state );
}


/**
 * registered system callback for protocol (debug) messages
 */
int callback_system_msg( ds485Handle_t dev, unsigned char level, const char *msg, va_list args)
{
    char buffer[512];

    vsnprintf(buffer, 512, msg, args);

    printf( "%s", buffer);
    return 0;
}

/******************************* MAIN *******************************/

static dsid_t    id = { .value[0] = 42 };

int main(int argc, char* argv[])
{
    unsigned int    yes = 1;
    unsigned int    no  = 0;
    ds485Handle_t    dev;
    static char* serialDevice;

    /* Last arg is device or file name. */
    serialDevice = strdup(argv[1]);

    dev    = ds485_api_open( serialDevice, &id );

    /* start ds485 protocol */
    if( dev == NULL )
    {
        printf( "ERROR initializing dS485 protocol\n");
        exit(2);
    }

    sem_init( &sem, 0, 0 );

//    ds485_api_ioctl( dev, IOC_BLOCKING_MODE, &no, sizeof( no ) );

    fdLog    = open( LOG_DEV, O_WRONLY | O_NONBLOCK );
    if( fdLog >= 0 ){
        ds485_api_ioctl( dev, IOC_SET_LOGGER_FD, &fdLog, sizeof(fdLog));
        printf( "logger activated\n" );
    }

    ds485_api_ioctl( dev, IOC_SET_BUS_CHANGE_CB, callback_bus_change, sizeof(callback_bus_change) );
    ds485_api_ioctl( dev, IOC_SET_RECEIVED_CB, callback_received, sizeof(callback_received) );
    ds485_api_ioctl( dev, IOC_SET_SENDER_CB, callback_sender, sizeof(callback_received) );
    ds485_api_ioctl( dev, IOC_SET_BUS_STATE_CB, callback_state, sizeof(callback_state) );
    /* register message handler */
    ds485_api_ioctl( dev, IOC_SET_MSG_CB, callback_system_msg, sizeof(callback_system_msg));

    /* configure software loopback */
    ds485_api_ioctl( dev, IOC_LOOPBACK_MODE, &yes, sizeof(yes));

    sem_wait( &sem );

    Ds485Container_t    packet =
    {
//        .address  = id,
//        .address  = DS485_BROADCAST_ADDR,
        .address  = { .value = {0x35, 0x04, 0x17, 0x5f, 0xe0, 0x00, 0x00, 0x1f, 0xff, 0xff, 0xff, 0xff} },
//        .address  = { .value = {0x35, 0x04, 0x17, 0x5f, 0xe0, 0x00, 0x00, 0x1f, 0xff, 0xff, 0xff, 0xfe} },
        .length   = 3,
        .data  = { 0x00, 0x90, 0x00 },
        .type = DSTELEGRAM_REQUEST
    };
    int i;
//    for( i=0; i < 200; i++ )
    {
    if( ds485_api_send_data( dev, &packet ) < 0 ){
        printf( "Writing of data failed\n" );
//        ds485_api_close( dev );
//        return    -1;
    }
    }
    printf( "wrote\n" );

/*    Ds485Packet_t    packet;
    if( ds485_api_recv_data( dev, &packet ) >= 0 ){
        int idx;
        for( idx = 0; idx < packet.len; idx++ ){
            printf( "%2x ", packet.data[idx] );
        }
        char    buf[40];
        sprintDsid( &packet.addr, buf );
        printf( "\nfrom %s\n", buf );
    }*/

/*    int i;
    for( i= 0; i < 100; i++ ){
        dsid_t    id[64];
        int        num;
        num    = ds485_api_get_bus_members( dev, id );
        while( num > 0 ){
            num--;
            char    buf[40];
            sprintDsid( &id[num], buf );
            printf( "%s\n", buf );
        }
        printf( "----\n" );
        sleep( 3 );
    }*/

    {
    int i;
    for( i= 0; i < 100; i++ ){
        BusState        state;
        state    = ds485_api_get_bus_state( dev );
//        printf( "%i\n", state );
        sleep( 3 );
    }
    }

    /* shutdown ds485 protocol */
    ds485_api_close( dev );

    return 0;
}
